import logging
import sys
import io
import os
import time
import datetime
from colorama import init, Fore, Style

init()

# --- Set UTF-8 Encoding for Console and Files ---
if sys.platform.startswith('win'):
    sys.stdout = io.TextIOWrapper(sys.stdout.buffer, encoding='utf-8')
    sys.stderr = io.TextIOWrapper(sys.stderr.buffer, encoding='utf-8')

# Define UnicodeSafeFormatter class
class UnicodeSafeFormatter(logging.Formatter):
    """Custom formatter to handle Unicode characters safely"""
    def format(self, record):
        try:
            return super().format(record)
        except UnicodeEncodeError:
            record.msg = record.msg.encode('utf-8', errors='replace').decode('utf-8')
            return super().format(record)

# Redirect print to logging with original text (including colors for terminal, stripped for log)
class StreamToLogger:
    def __init__(self, logger, log_level=logging.INFO):
        self.logger = logger
        self.log_level = log_level
        self.linebuf = ''
        # Create a duplicate of the original stdout to avoid closure issues
        self.original_stdout_fd = os.dup(sys.stdout.fileno())
        self.original_stdout = os.fdopen(self.original_stdout_fd, 'w', encoding='utf-8', newline='')

    def write(self, buf):
        for line in buf.rstrip().splitlines():
            # Print to terminal with original text (preserves colors)
            self.original_stdout.write(line + '\n')
            self.original_stdout.flush()
            # Remove ANSI color codes before logging to file (log only once)
            clean_line = line.replace('\033[31m', '').replace('\033[32m', '').replace('\033[0m', '')
            if clean_line.strip():  # Avoid logging empty lines
                self.logger.log(self.log_level, clean_line.rstrip())

    def flush(self):
        self.original_stdout.flush()

# Update logging setup
def setup_logging():
    log_dir = 'logs'
    if not os.path.exists(log_dir):
        os.makedirs(log_dir)  # Create logs/ folder if it doesn't exist
    log_file = os.path.join(log_dir, 'kite_bracket_orders.log')

    # Get the root logger and clear any existing handlers to prevent console output
    logger = logging.getLogger()
    logger.setLevel(logging.INFO)
    logger.handlers = []  # Clear all existing handlers (prevents default StreamHandler)

    # Add only the FileHandler
    file_handler = logging.FileHandler(log_file, encoding='utf-8')
    file_handler.setFormatter(UnicodeSafeFormatter('%(asctime)s [%(levelname)s] %(message)s'))
    logger.addHandler(file_handler)

    # Redirect print to logger (terminal and log file output)
    sys.stdout = StreamToLogger(logger, logging.INFO)

# --- Market Hours Check ---
def is_market_open():
    now = datetime.datetime.now().time()
    return datetime.time(9, 15) <= now <= datetime.time(15, 30)

def is_weekend():
    """Check if it's Saturday or Sunday"""
    return datetime.datetime.now().weekday() >= 5

def current_timestamp():
    """Return current timestamp as string"""
    return datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")

# --- Check if Order is Filled ---
def is_filled(kite, order_id):
    try:
        history = kite.order_history(order_id)
        for o in history:
            if o['status'] == 'COMPLETE' and o['filled_quantity'] == o['quantity']:
                return True
        return False
    except Exception as e:
        print(f"⚠️ is_filled error: {e}")
        return False

# --- Cancel Order Safely ---
def cancel_order_safe(kite, order_id):
    try:
        kite.cancel_order(variety=kite.VARIETY_REGULAR, order_id=order_id)
        print(f"🚫 Cancelled order: {order_id}")
    except Exception as e:
        print(f"⚠️ Cancel error: {e}")

# --- Retry Wrapper (Loop until success) ---
def retry_until_success(func, *args, interval=5):
    while True:
        try:
            if func(*args):
                return True
        except Exception as e:
            print(f"🔁 Retry error: {e}")
        time.sleep(interval)
